// Copyright 2009 - Paul S. Heidmann
//
// Unless changed, in writing, by Paul S. Heidmann, this code is
// governed by the GPL, version 3. The terms of this copyright
// are found in the file throughputTest.cpp, included in this
// distribution.
//

#ifndef CONTENTIONPOINT_HPP
#define CONTENTIONPOINT_HPP

#include <boost/thread/mutex.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/date_time/time.hpp>

#include "gaussianRandomVariable.hpp"
#include "workModel.hpp"
#include "timeProfile.hpp"

class contentionPoint
{
public:

  contentionPoint( const gaussianRandomVariable& produceWorkVar,
		   const gaussianRandomVariable& consumeWorkVar,
		   const unsigned long capacity,
		   const unsigned testTimeInSeconds );
  ~contentionPoint();

  void startTest( void );
  const bool produce( timeProfile& p );
  const bool consume( timeProfile& p );

  const unsigned long getNumConsumptions( void ) const;

private:

  const unsigned testTime;
  const unsigned long size;
  boost::posix_time::ptime startTime;
  workModel produceWork;
  workModel consumeWork;
  boost::mutex lock;
  bool testStarted;
  bool testDone;
  unsigned long numConsumptions;
  unsigned long numItemsAvailable;

  contentionPoint();
  contentionPoint( const contentionPoint& );
  const bool checkTestDone( void );
};

#endif
