// Copyright 2009 - Paul S. Heidmann
//
// Unless changed, in writing, by Paul S. Heidmann, this code is
// governed by the GPL, version 3. The terms of this copyright
// are found in the file throughputTest.cpp, included in this
// distribution.
//

#include "gnuPlotter.hpp"

#include <stdio.h>

#include "throughputException.hpp"

#ifdef WIN32
#define POPEN _popen
#define PCLOSE _pclose
#else
#define POPEN popen
#define PCLOSE pclose
#endif

gnuPlotter::gnuPlotter( const std::string inFileName ) :
#ifdef WIN32
  PATH_TO_GNUPLOT( "c:\\cygwin\\bin\\gnuplot.exe" ),
#else
  PATH_TO_GNUPLOT( "/usr/local/bin/gnuplot" ),
#endif
  dataFile( inFileName )
{
  this->constructPlotCommands();
}

gnuPlotter::~gnuPlotter()
{
}

void gnuPlotter::operator()( void )
{
  FILE* plotPipe( POPEN( this->PATH_TO_GNUPLOT.c_str(), "w" ) );
  if( plotPipe == NULL )
    throw throughputException( "Cannot execute gnuplot." );
  fprintf( plotPipe, "%s\n", this->gnuPlotCommands.str().c_str() );
  PCLOSE( plotPipe );
}

void gnuPlotter::constructPlotCommands( void )
{
  this->gnuPlotCommands << "set pm3d map"
			<< std::endl
			<< "set terminal png"
			<< std::endl
			<< "set output \""
			<< this->dataFile
			<< ".png\""
			<< std::endl
			<< "splot \""
			<< this->dataFile
			<< "\" with pm3d"
			<< std::endl
			<< "quit"
			<< std::endl;

}
