// Copyright 2009 - Paul S. Heidmann
//
// Unless changed, in writing, by Paul S. Heidmann, this code is
// governed by the GPL, version 3. The terms of this copyright
// are found in the file throughputTest.cpp, included in this
// distribution.
//

#include "ioDevice.hpp"

#include "timeProfileConsts.hpp"
#include "throughputException.hpp"

static unsigned char ioDeviceSpace[sizeof( ioDevice )];
static bool ioDeviceInitialized( false );

ioDevice* ioDevice::getInstance( void )
{
	if( !ioDeviceInitialized )
	{
		ioDevice* tmp( new( static_cast<void*>( ioDeviceSpace ) ) ioDevice );
		ioDeviceInitialized = true;
	}
	return( reinterpret_cast<ioDevice*>( ioDeviceSpace ) );
}

void ioDevice::doIo(
	const unsigned long numNanoSecsIo,
	timeProfile& profile )
{
	profile.insertProfilePoint( IO_MODEL_ID | 1 );

	bool okToDoIo( false );
	while( !okToDoIo )
	{
		{
			boost::mutex::scoped_lock localLock( this->lock );
			if( this->numThreadsDoingIo < 0 )
				throw( throughputException( "ERROR: Fewer than zero threads doing I/O." ) );
			if( this->numThreadsDoingIo > this->maxNumIoThreads )
				throw( throughputException( "ERROR: Too many threads doing I/O." ) );
			if( this->numThreadsDoingIo < this->maxNumIoThreads )
			{
				okToDoIo = true;
				++(this->numThreadsDoingIo);
			}
		}
		boost::this_thread::sleep(
			boost::posix_time::microseconds( this->pollingIntervalMicroSecs ) );
	}

	profile.insertProfilePoint( IO_MODEL_ID | 2 );

	boost::this_thread::sleep( boost::posix_time::microseconds( numNanoSecsIo / 1000 ) );
	{
		boost::mutex::scoped_lock localLock( this->lock );
		--(this->numThreadsDoingIo);
	}

	profile.insertProfilePoint( IO_MODEL_ID | 3 );
}

ioDevice::ioDevice() :
	pollingIntervalMicroSecs( 5000 ),
	numThreadsDoingIo( 0 ),
	maxNumIoThreads( 3 )
{
}
