// Copyright 2009 - Paul S. Heidmann
//
// Unless changed, in writing, by Paul S. Heidmann, this code is
// governed by the GPL, version 3. The terms of this copyright
// are found in the file throughputTest.cpp, included in this
// distribution.
//

#include "profileAgregate.hpp"

#include <iomanip>

profileAgregate::profileResultsType::profileResultsType( 
	throughputControl const * const theControl,
	timeProfile const * const theProfile ) :
control( theControl ),
	profile( theProfile )
{
}

profileAgregate::profileResultsType::profileResultsType( const profileResultsType& copy ) :
control( copy.control ),
	profile( copy.profile )
{
}

profileAgregate::profileAgregate()
{
	this->agregate.clear();
}

profileAgregate::profileAgregate( const profileAgregate& copy ) :
agregate( copy.agregate )
{
}

profileAgregate::~profileAgregate()
{
	std::list<profileResultsType>::const_iterator i;
	for( i = this->agregate.begin();
		i != this->agregate.end();
		++i )
	{
		delete( i->control );
		delete( i->profile );
	}
}

void profileAgregate::insertProfileResults( const throughputControl& controlParams,
	const timeProfile& profResults )
{
	profileResultsType newResult( new throughputControl( controlParams ),
		new timeProfile( profResults ) );
	this->agregate.push_back( newResult );
}

const std::list<profileAgregate::profileResultsType>&
	profileAgregate::getAgregatedResults( void ) const
{
	return( this->agregate );
}

void profileAgregate::printHeader( std::ostream& str ) const
{
	throughputControl::printHeader( str );
	str << ',';
	str << "threadId,"
		<< "locationId,"
		<< "sec,"
		<< "nsec,"
		<< "timeToPrev"
		<< std::endl;
}

std::ostream& operator<<( std::ostream& str, const profileAgregate& obj )
{
	obj.printHeader( str );
	std::list<profileAgregate::profileResultsType>::const_iterator i;
	for( i = obj.getAgregatedResults().begin();
		 i != obj.getAgregatedResults().end();
		 ++i )
	{
		std::vector<timeProfile::timeProfilePoint>::const_iterator j;
		std::vector<timeProfile::timeProfilePoint>::const_iterator
			prevJ( i->profile->getProfilePoints().begin() );
		for( j = i->profile->getProfilePoints().begin();
			 j != i->profile->getProfilePoints().end();
			 ++j )
		{
			str << (*i->control)
				<< ','
				<< i->profile->getThreadId()
				<< ','
				<< "0x" << std::hex << j->locationId
				<< std::dec
				<< ','
				<< j->time
				<< ',';
			if( j == i->profile->getProfilePoints().begin() )
				str << "N/A";
			else
				str << (j->time - prevJ->time);
			str << std::endl;
			prevJ = j;
		}
	}
	return( str );
}
