// Copyright 2009 - Paul S. Heidmann
//
// Unless changed, in writing, by Paul S. Heidmann, this code is
// governed by the GPL, version 3. The terms of this copyright
// are found in the file throughputTest.cpp, included in this
// distribution.
//

#include "threadAgregate.hpp"

threadAgregate::threadAgregate( const workModel& producerWorkModel,
				const workModel& consumerWorkModel,
				contentionPoint* sharedRes,
				const unsigned numProducers,
				const unsigned numConsumers ) :
  numProd( numProducers ),
  numCons( numConsumers ),
  prodWork( producerWorkModel ),
  consWork( consumerWorkModel ),
  theSharedResource( sharedRes )
{
  this->producerObjList.clear();
  this->consumerObjList.clear();
  this->createAllThreadObjs();
}

threadAgregate::~threadAgregate()
{
  std::list<producer*>::iterator i;
  std::list<consumer*>::iterator j;

  for( i = this->producerObjList.begin();
       i != this->producerObjList.end();
       ++i )
    delete( *i );
  for( j = this->consumerObjList.begin();
       j != this->consumerObjList.end();
       ++j )
    delete( *j );
}

void threadAgregate::spawnThreads( void )
{
  std::list<producer*>::iterator i;
  std::list<consumer*>::iterator j;

  for( i = this->producerObjList.begin();
       i != this->producerObjList.end();
       ++i )
    this->producerThreads.create_thread( **i );
  for( j = this->consumerObjList.begin();
       j != this->consumerObjList.end();
       ++j )
    this->consumerThreads.create_thread( **j );
}

void threadAgregate::waitForCompletion( void )
{
  this->producerThreads.join_all();
  this->consumerThreads.join_all();
}

const std::list<producer*>& threadAgregate::getProducers( void ) const
{
  return( this->producerObjList );
}

const std::list<consumer*>& threadAgregate::getConsumers( void ) const
{
  return( this->consumerObjList );
}

void threadAgregate::createAllThreadObjs( void )
{
  for( unsigned i = 0; i < this->numProd; ++i )
    this->producerObjList.push_back( new producer( this->theSharedResource,
						   this->prodWork ) );
  for( unsigned j = 0; j < this->numCons; ++j )
    this->consumerObjList.push_back( new consumer( this->theSharedResource,
						   this->consWork ) );
}
