// Copyright 2009 - Paul S. Heidmann
//
// Unless changed, in writing, by Paul S. Heidmann, this code is
// governed by the GPL, version 3. The terms of this copyright
// are found in the file throughputTest.cpp, included in this
// distribution.
//

#ifndef THREADAGREGATE_HPP
#define THREADAGREGATE_HPP

#include <list>
#include <boost/thread/thread.hpp>
#include "workModel.hpp"
#include "contentionPoint.hpp"
#include "producer.hpp"
#include "consumer.hpp"

class threadAgregate
{
public:

  threadAgregate( const workModel& producerWorkModel,
		  const workModel& consumerWorkModel,
		  contentionPoint* sharedRes,
		  const unsigned numProducers,
		  const unsigned numConsumers );
  ~threadAgregate();

  void spawnThreads( void );
  void waitForCompletion( void );
  const std::list<producer*>& getProducers( void ) const;
  const std::list<consumer*>& getConsumers( void ) const;

private:

  const unsigned numProd;
  const unsigned numCons;
  const workModel prodWork;
  const workModel consWork;
  contentionPoint * const theSharedResource;
  std::list<producer*> producerObjList;
  std::list<consumer*> consumerObjList;
  boost::thread_group producerThreads;
  boost::thread_group consumerThreads;

  threadAgregate();
  threadAgregate( const threadAgregate& );
  void createAllThreadObjs( void );
};

#endif
