// Copyright 2009 - Paul S. Heidmann
//
// Unless changed, in writing, by Paul S. Heidmann, this code is
// governed by the GPL, version 3. The terms of this copyright
// are found in the file throughputTest.cpp, included in this
// distribution.
//

#include "throughputConfigFile.hpp"

#include <vector>
#include <sstream>
#include <fstream>
#include <boost/tokenizer.hpp>

#include "throughputException.hpp"

throughputConfigFile::throughputConfigFile( const std::string& confFileName ) :
  fileName( confFileName ),
  numParamsPerLine( 16 )
{
  this->testControlList.clear();
  this->readInConfigFile();
}

throughputConfigFile::throughputConfigFile( const throughputConfigFile& copy ) :
  fileName( copy.fileName ),
  testControlList( copy.testControlList ),
  numParamsPerLine( copy.numParamsPerLine )
{
}

throughputConfigFile::~throughputConfigFile()
{
}

const std::list<throughputControl>& throughputConfigFile::getTestControls( void ) const
{
  return( this->testControlList );
}

void throughputConfigFile::readInConfigFile( void )
{
  std::ifstream inF( this->fileName.c_str() );
  if( !(inF.good()) )
    throw( throughputException( "Cannot open config file." ) );

  const boost::char_separator<char> configLineSeperator( "," );
  while( !(inF.eof()) )
  {
    std::string curLine;
    curLine.clear();
    std::getline( inF, curLine );
    if( curLine.length() < 3 )
      continue;
    boost::tokenizer<boost::char_separator<char> > tok( curLine, configLineSeperator );
    boost::tokenizer<boost::char_separator<char> >::iterator i( tok.begin() );
    std::vector<unsigned long> paramsList;
    paramsList.clear();
    while( i != tok.end() )
    {
      std::istringstream pStream( *i );
      unsigned long p( 0 );
      pStream >> p;
      paramsList.push_back( p );
      ++i;
    }
    if( paramsList.size() != this->numParamsPerLine )
      throw( throughputException( "Incorrect number of parameters on config line." ) );

    throughputControl curControl( paramsList.at( 0 ),
				  paramsList.at( 1 ),
				  paramsList.at( 2 ),
				  paramsList.at( 3 ),
				  paramsList.at( 4 ),
				  paramsList.at( 5 ),
				  paramsList.at( 6 ),
				  paramsList.at( 7 ),
				  paramsList.at( 8 ),
				  paramsList.at( 9 ),
				  paramsList.at( 10 ),
				  paramsList.at( 11 ),
				  paramsList.at( 12 ),
				  paramsList.at( 13 ),
				  paramsList.at( 14 ),
				  paramsList.at( 15 ) );
    this->testControlList.push_back( curControl );
  }
}
