// Copyright 2009 - Paul S. Heidmann
//
// Unless changed, in writing, by Paul S. Heidmann, this code is
// governed by the GPL, version 3. The terms of this copyright
// are found in the file throughputTest.cpp, included in this
// distribution.
//

#include "throughputControl.hpp"
#include "throughputException.hpp"

throughputControl::throughputControl( const unsigned long nonCriticalProducerWorkItersMean,
				      const unsigned long nonCriticalProducerWorkItersStdDev,
				      const unsigned long nonCriticalProducerWorkIoWaitMean,
				      const unsigned long nonCriticalProducerWorkIoWaitStdDev,
				      const unsigned long nonCriticalConsumerWorkItersMean,
				      const unsigned long nonCriticalConsumerWorkItersStdDev,
				      const unsigned long nonCriticalConsumerWorkIoWaitMean,
				      const unsigned long nonCriticalConsumerWorkIoWaitStdDev,
				      const unsigned long criticalProducerWorkItersMean,
				      const unsigned long criticalProducerWorkItersStdDev,
				      const unsigned long criticalConsumerWorkItersMean,
				      const unsigned long criticalConsumerWorkItersStdDev,
				      const unsigned long resourceCapacity,
				      const unsigned numProducerThreads,
				      const unsigned numConsumerThreads,
				      const unsigned testTime ) :
  nonCritProducerWorkItersMean( nonCriticalProducerWorkItersMean ),
  nonCritProducerWorkItersStdDev( nonCriticalProducerWorkItersStdDev ),
  nonCritProducerWorkIoWaitMean( nonCriticalProducerWorkIoWaitMean ),
  nonCritProducerWorkIoWaitStdDev( nonCriticalProducerWorkIoWaitStdDev ),
  nonCritConsumerWorkItersMean( nonCriticalConsumerWorkItersMean ),
  nonCritConsumerWorkItersStdDev( nonCriticalConsumerWorkItersStdDev ),
  nonCritConsumerWorkIoWaitMean( nonCriticalConsumerWorkIoWaitMean ),
  nonCritConsumerWorkIoWaitStdDev( nonCriticalConsumerWorkIoWaitStdDev ),
  critProducerWorkItersMean( criticalProducerWorkItersMean ),
  critProducerWorkItersStdDev( criticalProducerWorkItersStdDev ),
  critConsumerWorkItersMean( criticalConsumerWorkItersMean ),
  critConsumerWorkItersStdDev( criticalConsumerWorkItersStdDev ),
  resourceCap( resourceCapacity ),
  numProdThreads( numProducerThreads ),
  numConsThreads( numConsumerThreads ),
  tstTime( testTime ),
  numConsumptions( 0 )
{
}

throughputControl::throughputControl( const throughputControl& copy ) :
  nonCritProducerWorkItersMean( copy.nonCritProducerWorkItersMean ),
  nonCritProducerWorkItersStdDev( copy.nonCritProducerWorkItersStdDev ),
  nonCritProducerWorkIoWaitMean( copy.nonCritProducerWorkIoWaitMean ),
  nonCritProducerWorkIoWaitStdDev( copy.nonCritProducerWorkIoWaitStdDev ),
  nonCritConsumerWorkItersMean( copy.nonCritConsumerWorkItersMean ),
  nonCritConsumerWorkItersStdDev( copy.nonCritConsumerWorkItersStdDev ),
  nonCritConsumerWorkIoWaitMean( copy.nonCritConsumerWorkIoWaitMean ),
  nonCritConsumerWorkIoWaitStdDev( copy.nonCritConsumerWorkIoWaitStdDev ),
  critProducerWorkItersMean( copy.critProducerWorkItersMean ),
  critProducerWorkItersStdDev( copy.critProducerWorkItersStdDev ),
  critConsumerWorkItersMean( copy.critConsumerWorkItersMean ),
  critConsumerWorkItersStdDev( copy.critConsumerWorkItersStdDev ),
  resourceCap( copy.resourceCap ),
  numProdThreads( copy.numProdThreads ),
  numConsThreads( copy.numConsThreads ),
  tstTime( copy.tstTime ),
  numConsumptions( copy.numConsumptions )
{
}

throughputControl::~throughputControl()
{
}

void throughputControl::setNumConsumptions( const unsigned long numCons )
{
  this->numConsumptions = numCons;
}

const unsigned long throughputControl::getNonCritProducerWorkItersMean( void ) const
{
  return( this->nonCritProducerWorkItersMean );
}

const unsigned long throughputControl::getNonCritProducerWorkItersStdDev( void ) const
{
  return( this->nonCritProducerWorkItersStdDev );
}

const unsigned long throughputControl::getNonCritProducerWorkIoWaitMean( void ) const
{
  return( this->nonCritProducerWorkIoWaitMean );
}

const unsigned long throughputControl::getNonCritProducerWorkIoWaitStdDev( void ) const
{
  return( this->nonCritProducerWorkIoWaitStdDev );
}

const unsigned long throughputControl::getNonCritConsumerWorkItersMean( void ) const
{
  return( this->nonCritConsumerWorkItersMean );
}

const unsigned long throughputControl::getNonCritConsumerWorkItersStdDev( void ) const
{
  return( this->nonCritConsumerWorkItersStdDev );
}

const unsigned long throughputControl::getNonCritConsumerWorkIoWaitMean( void ) const
{
  return( this->nonCritConsumerWorkIoWaitMean );
}

const unsigned long throughputControl::getNonCritConsumerWorkIoWaitStdDev( void ) const
{
  return( this->nonCritConsumerWorkIoWaitStdDev );
}

const unsigned long throughputControl::getCritProducerWorkItersMean( void ) const
{
  return( this->critProducerWorkItersMean );
}

const unsigned long throughputControl::getCritProducerWorkItersStdDev( void ) const
{
  return( this->critProducerWorkItersStdDev );
}

const unsigned long throughputControl::getCritConsumerWorkItersMean( void ) const
{
  return( this->critConsumerWorkItersMean );
}

const unsigned long throughputControl::getCritConsumerWorkItersStdDev( void ) const
{
  return( this->critConsumerWorkItersStdDev );
}

const unsigned long throughputControl::getResourceCap( void ) const
{
  return( this->resourceCap );
}

const unsigned throughputControl::getNumProdThreads( void ) const
{
  return( this->numProdThreads );
}

const unsigned throughputControl::getNumConsThreads( void ) const
{
  return( this->numConsThreads );
}

const unsigned throughputControl::getTstTime( void ) const
{
  return( this->tstTime );
}

unsigned long throughputControl::getNumConsumptions( void ) const
{
  return( this->numConsumptions );
}

void throughputControl::printHeader( std::ostream& str )
{
  str << "nonCtitProdItersMean,"
      << "nonCtitProdItersStdDev,"
      << "nonCtitProdIoNanoMean,"
      << "nonCtitProdIoNanoStdDev,"
      << "nonCtitConsItersMean,"
      << "nonCtitConsItersStdDev,"
      << "nonCtitConsIoNanoMean,"
      << "nonCtitConsIoNanoStdDev,"
      << "critProdItersMean,"
      << "critProdItersStdDev,"
      << "critConsItersMean,"
      << "critConsItersStdDev,"
      << "resourceCap,"
      << "numProdThreads,"
      << "numConsThreads,"
      << "testTime,"
      << "numConsuptions";
}

void throughputControl::dumpGnuPlotFileLine( std::ofstream& gnuPlotFile )
{
  gnuPlotFile << this->getNumProdThreads() << ' '
	      << this->getNumConsThreads() << ' '
	      << this->getNumConsumptions()
	      << std::endl;
}

std::ostream& operator<<( std::ostream& str, const throughputControl& obj )
{
  str << obj.getNonCritProducerWorkItersMean() << ','
      << obj.getNonCritProducerWorkItersStdDev() << ','
      << obj.getNonCritProducerWorkIoWaitMean() << ','
      << obj.getNonCritProducerWorkIoWaitStdDev() << ','
      << obj.getNonCritConsumerWorkItersMean() << ','
      << obj.getNonCritConsumerWorkItersStdDev() << ','
      << obj.getNonCritConsumerWorkIoWaitMean() << ','
      << obj.getNonCritConsumerWorkIoWaitStdDev() << ','
      << obj.getCritProducerWorkItersMean() << ','
      << obj.getCritProducerWorkItersStdDev() << ','
      << obj.getCritConsumerWorkItersMean() << ','
      << obj.getCritConsumerWorkItersStdDev() << ','
      << obj.getResourceCap() << ','
      << obj.getNumProdThreads() << ','
      << obj.getNumConsThreads() << ','
      << obj.getTstTime() << ','
      << obj.getNumConsumptions();
  return( str );
}
