// Copyright 2009 - Paul S. Heidmann
//
// Unless changed, in writing, by Paul S. Heidmann, this code is
// governed by the GPL, version 3. The terms of this copyright
// are found in the file throughputTest.cpp, included in this
// distribution.
//

#ifndef THROUGHPUTCONTROL_HPP
#define THROUGHPUTCONTROL_HPP

#include <fstream>

class throughputControl
{
public:

  throughputControl( const unsigned long nonCriticalProducerWorkItersMean,
		     const unsigned long nonCriticalProducerWorkItersStdDev,
		     const unsigned long nonCriticalProducerWorkIoWaitMean,
		     const unsigned long nonCriticalProducerWorkIoWaitStdDev,
		     const unsigned long nonCriticalConsumerWorkItersMean,
		     const unsigned long nonCriticalConsumerWorkItersStdDev,
		     const unsigned long nonCriticalConsumerWorkIoWaitMean,
		     const unsigned long nonCriticalConsumerWorkIoWaitStdDev,
		     const unsigned long criticalProducerWorkItersMean,
		     const unsigned long criticalProducerWorkItersStdDev,
		     const unsigned long criticalConsumerWorkItersMean,
		     const unsigned long criticalConsumerWorkItersStdDev,
		     const unsigned long resourceCapacity,
		     const unsigned numProducerThreads,
		     const unsigned numConsumerThreads,
		     const unsigned testTime );
  throughputControl( const throughputControl& copy );
  ~throughputControl();

  void setNumConsumptions( const unsigned long numCons );

  const unsigned long getNonCritProducerWorkItersMean( void ) const;
  const unsigned long getNonCritProducerWorkItersStdDev( void ) const;
  const unsigned long getNonCritProducerWorkIoWaitMean( void ) const;
  const unsigned long getNonCritProducerWorkIoWaitStdDev( void ) const;
  const unsigned long getNonCritConsumerWorkItersMean( void ) const;
  const unsigned long getNonCritConsumerWorkItersStdDev( void ) const;
  const unsigned long getNonCritConsumerWorkIoWaitMean( void ) const;
  const unsigned long getNonCritConsumerWorkIoWaitStdDev( void ) const;
  const unsigned long getCritProducerWorkItersMean( void ) const;
  const unsigned long getCritProducerWorkItersStdDev( void ) const;
  const unsigned long getCritConsumerWorkItersMean( void ) const;
  const unsigned long getCritConsumerWorkItersStdDev( void ) const;
  const unsigned long getResourceCap( void ) const;
  const unsigned getNumProdThreads( void ) const;
  const unsigned getNumConsThreads( void ) const;
  const unsigned getTstTime( void ) const;
  unsigned long getNumConsumptions( void ) const;

  static void printHeader( std::ostream& str );
  void dumpGnuPlotFileLine( std::ofstream& gnuPlotFile );

private:

  const unsigned long nonCritProducerWorkItersMean;
  const unsigned long nonCritProducerWorkItersStdDev;
  const unsigned long nonCritProducerWorkIoWaitMean;
  const unsigned long nonCritProducerWorkIoWaitStdDev;
  const unsigned long nonCritConsumerWorkItersMean;
  const unsigned long nonCritConsumerWorkItersStdDev;
  const unsigned long nonCritConsumerWorkIoWaitMean;
  const unsigned long nonCritConsumerWorkIoWaitStdDev;
  const unsigned long critProducerWorkItersMean;
  const unsigned long critProducerWorkItersStdDev;
  const unsigned long critConsumerWorkItersMean;
  const unsigned long critConsumerWorkItersStdDev;
  const unsigned long resourceCap;
  const unsigned numProdThreads;
  const unsigned numConsThreads;
  const unsigned tstTime;
  unsigned long numConsumptions;

  throughputControl();
};

std::ostream& operator<<( std::ostream& str, const throughputControl& obj );

#endif
