// Copyright 2009 - Paul S. Heidmann
//
// Unless changed, in writing, by Paul S. Heidmann, this code is
// governed by the GPL, version 3. The terms of this copyright
// are found in the file throughputTest.cpp, included in this
// distribution.
//

#include "workModel.hpp"

#include <math.h>
#include <boost/random/uniform_real.hpp>

#include "ioDevice.hpp"
#include "timeProfileConsts.hpp"

workModel::workModel( const gaussianRandomVariable& workIterationsVar ) :
	hasIoVar( false ),
	uniReal( 0.0, 1.0 ),
	randObj( randNumGen, uniReal )
{
	this->iterVar.reset( new gaussianRandomVariable( workIterationsVar ) );
}

workModel::workModel(
	const gaussianRandomVariable& workIterationsVar,
	const gaussianRandomVariable& ioNanoSecsWaitVar ) :
	hasIoVar( true ),
	uniReal( 0.0, 1.0 ),
	randObj( randNumGen, uniReal )
{
	this->iterVar.reset( new gaussianRandomVariable( workIterationsVar ) );
	this->ioVar.reset( new gaussianRandomVariable( ioNanoSecsWaitVar ) );
}

workModel::workModel( const workModel& copy ) :
	hasIoVar( copy.hasIoVar ),
	uniReal( copy.uniReal ),
	randObj( copy.randObj )
{
	this->iterVar.reset( new gaussianRandomVariable( *(copy.iterVar) ) );
	if( this->hasIoVar )
		this->ioVar.reset( new gaussianRandomVariable( *(copy.ioVar) ) );
}

workModel::~workModel()
{
}

double workModel::doWork( timeProfile& prof )
{
	prof.insertProfilePoint( WORK_MODEL_ID | 1 );

	double x;
	const unsigned long numIters( this->iterVar->getSample() );
	x = 1.0;
	for( unsigned long i = 0; i < numIters; ++i )
	{
		x = sin( this->randObj() * x );
		x *= x;
		if( x < 0.01 )
			x = 0.5;
		if( x > 0.9 )
			x = 0.5;
	}

	prof.insertProfilePoint( WORK_MODEL_ID | 2 );

	if( this->hasIoVar )
	{
		const unsigned long ioWaitNanoSecs( this->ioVar->getSample() );

		ioDevice::getInstance()->doIo( ioWaitNanoSecs, prof );
	}

	prof.insertProfilePoint( WORK_MODEL_ID | 3 );

	return( x );
}
